import re
from pathlib import Path
from typing import Match, List, Any


def detect_mul(data: str) -> list[Match]:
    return re.findall(r"mul\((\d{1,3}),(\d{1,3})\)", data)


def compute_multiplication(data: str) -> int:
    result = 0
    for match in detect_mul(data):
        left = int(match[0])
        right = int(match[1])
        result += left * right
    return result


def find_activated_parts(data: str) -> str:
    res = []

    start_match = re.search(r"^.*?don't\(\)", data)
    if start_match:
        start_data = start_match[0]
        res.append(start_data)
        data = data.removeprefix(start_data)

    middle_match = re.search(r"^.*don't\(\)", data)
    if middle_match:
        middle_data = middle_match[0]
        matches = re.findall(r"do\(\).*?don't\(\)", middle_data)
        res.extend(matches)
        data = data.removeprefix(middle_data)

    end_match = re.search(r"do\(\).*$", data)
    if end_match:
        end_data = end_match[0]
        res.append(end_data)

    return "".join(res)


def main(input_file: Path) -> int:
    input_data = ""
    for line in input_file.open():
        input_data += line.strip()
    activated_parts = find_activated_parts(input_data)
    return compute_multiplication(activated_parts)


if __name__ == "__main__":
    file = Path(__file__).parent / "input-data"
    print(main(file))
